/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * MainFrame.java
 *
 * Created on 26.09.2010, 13:27:35
 */
package fontcreator;

import com.one.PNGFile;
import com.one.TextProcessor;
import com.vmx.ProgressCallback;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import javax.swing.ImageIcon;
import javax.swing.UIManager;

/**
 *
 * @author User
 */
public class MainFrame extends javax.swing.JFrame implements ProgressCallback, Runnable
{
	private static final int ROWS_IN_BLOCK = 16;
	private static final int COLS_IN_BLOCK = 16;

	private static final int CHARSET_UNICODE = -2;
	private static final int CHARSET_ALL_CODE_PAGES = -1;

	private static final String SEPARATOR = System.getProperty("file.separator");
	private static final String SUFFIX = ".zip";

	private GraphicsEnvironment env;
	private Font font;
	private FontMetrics metrics;
	private RenderingHints clearType;
	private String folder;
	private int charset;

	private Thread thread;
	protected boolean enabled;

	/** Creates new form MainFrame */
	public MainFrame()
	{
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Throwable t)
		{
		}

		initComponents();
		setLocationRelativeTo(null);

		env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		String[] families = env.getAvailableFontFamilyNames();

		for(int i = 0; i < families.length; i++)
		{
			cbFamily.addItem(families[i]);
		}

		for(int i = 0; i < Encoder.cpnames.length; i++)
		{
			cbCharset.addItem(Encoder.cpnames[i] + " only");
		}
	}

	public void updatePreview()
	{
		if(thread != null)
		{
			return;
		}

		String path = tfPath.getText();

		if(folder != null && path.endsWith(folder))
		{
			path = path.substring(0, path.length() - folder.length());
		}
		
		String family = (String)cbFamily.getSelectedItem();
		int size = ((Integer)spSize.getValue()).intValue();
		int style = Font.PLAIN;

		if(chBold.isSelected())
		{
			style |= Font.BOLD;
		}

		if(chItalic.isSelected())
		{
			style |= Font.ITALIC;
		}

		font = new Font(family, style, size);
		metrics = lPreview.getFontMetrics(font);

		folder = font.getPSName();
		char[] cs = "\\/:*?\"<>|. ".toCharArray();

		for(int i = 0; i < cs.length; i++)
		{
			folder = folder.replace(cs[i], '-');
		}

//		family = folder.toLowerCase();
//
//		if((style & Font.BOLD) != 0 && family.indexOf("bold") < 0)
//		{
//			folder += "-Bold";
//		}
//
//		if((style & Font.ITALIC) != 0 && family.indexOf("italic") < 0)
//		{
//			folder += "-Italic";
//		}

		charset = cbCharset.getSelectedIndex() - 2;

		folder += "-";

		if(charset == CHARSET_UNICODE)
		{
			folder += "unicode";
		}
		else if(charset == CHARSET_ALL_CODE_PAGES)
		{
			folder += "all";
		}
		else
		{
			folder += Encoder.cpnames[charset];
		}

		folder += "-" + Integer.toString(font.getSize());

		if(chClearType.isSelected())
		{
			if(clearType == null)
			{
				clearType = new RenderingHints(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
			}

			//folder += "-aa";
		}
		else
		{
			clearType = null;
		}

		folder += SUFFIX;

		if(!path.endsWith(SEPARATOR))
		{
			path += SEPARATOR;
		}

		tfPath.setText(path + folder);

		String sample = TextProcessor.unescapeString(tfSample.getText());

		if(sample.length() > 0)
		{
			int width = metrics.stringWidth(sample);
			int height = metrics.getMaxAscent() + metrics.getMaxDescent();

			BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
			Graphics2D g = image.createGraphics();

			if(clearType != null)
			{
				g.setRenderingHints(clearType);
			}

			g.setFont(font);

			if(clearType != null)
			{
				g.setColor(Color.WHITE);
				g.drawString(sample, 0, metrics.getMaxAscent());

				int[] rgb = new int[width * height];
				image.getRGB(0, 0, width, height, rgb, 0, width);

				int color = lPreview.getForeground().getRGB() & 0xFFFFFF;

				for(int j = 0; j < rgb.length; j++)
				{
					rgb[j] = (((((rgb[j] >> 16) & 0xFF) + ((rgb[j] >> 7) & 0x1FE) + (rgb[j] & 0xFF)) << 22) & 0xFF000000) | color;
				}
				
				image.setRGB(0, 0, width, height, rgb, 0, width);
			}
			else
			{
				g.setColor(lPreview.getForeground());
				g.drawString(sample, 0, metrics.getMaxAscent());
			}

			lPreview.setIcon(new ImageIcon(image));
		}
	}

	public void run()
	{
		btnStart.setText("Stop");

		try
		{
			String path = tfPath.getText();

//			if(!path.endsWith(SEPARATOR))
//			{
//				path += SEPARATOR;
//			}

//			System.out.println(path);

//			File file = new File(path);
//
//			if(!file.exists())
//			{
//				file.mkdirs();
//			}

			ZipOutputStream zip = new ZipOutputStream(new FileOutputStream(path));
			zip.setLevel(Deflater.BEST_COMPRESSION);

			int fh = metrics.getAscent() + metrics.getDescent();
			int offset = metrics.getAscent();

			int width, height;
			String imgname;

			char[] source;

			if(charset == CHARSET_UNICODE)
			{
				source = new char[65536];

				for(int i = 0; i < 65536; i++)
				{
					source[i] = (char)i;
				}
			}
			else if(charset == CHARSET_ALL_CODE_PAGES)
			{
				source = Encoder.getFullSet();
			}
			else
			{
				source = new char[256];

				for(int i = 0; i < 128; i++)
				{
					source[i] = (char)i;
				}

				for(int i = 128; i < 256; i++)
				{
					source[i] = Encoder.codepages[charset][i - 128];
				}
			}

			int blocksize = COLS_IN_BLOCK * ROWS_IN_BLOCK;
			int numblocks = (source.length + blocksize - 1) / blocksize;
			int lastblock = numblocks - 1;

			char[][] chars = new char[ROWS_IN_BLOCK][COLS_IN_BLOCK];
			int[][] cws = new int[ROWS_IN_BLOCK][COLS_IN_BLOCK];
			int cx, cy;
			int cw;

			height = fh * ROWS_IN_BLOCK;

			ByteArrayOutputStream ini = new ByteArrayOutputStream();
			PrintStream ps = new PrintStream(ini);
			StringBuffer buf = new StringBuffer();

			ps.println("[Info]");
			ps.println();

			ps.println("Face\t=\t" + font.getFontName());
			ps.println("Size\t=\t" + font.getSize());
			ps.println();

			ps.println("Ascent\t=\t" + metrics.getAscent());
			ps.println("Descent\t=\t" + metrics.getDescent());
			ps.println("Leading\t=\t" + metrics.getLeading());
			ps.println();

			ByteArrayOutputStream bin = new ByteArrayOutputStream();
			DataOutputStream binout = new DataOutputStream(bin);

			binout.writeUTF(font.getFontName());
			binout.writeShort(font.getSize());

			binout.writeShort(metrics.getAscent());
			binout.writeShort(metrics.getDescent());
			binout.writeShort(metrics.getLeading());
			
			binout.writeShort(numblocks);

			short[][] srcindex = new short[256][256];
			
			int hi, lo;
			
			for(hi = 0; hi < 256; hi++)
			{
				for(lo = 0; lo < 256; lo++)
				{
					srcindex[hi][lo] = -1;
				}
			}
			
			short[][] srcx = new short[256][256];
			short[][] srcy = new short[256][256];
			short[][] widths = new short[256][256];

			BufferedImage image;
			Graphics2D g;

			ByteArrayOutputStream png = new ByteArrayOutputStream();
			DataOutputStream pngout = new DataOutputStream(png);

			char[] c = new char[1];

			int index;
			int x, y;

			pbTotal.setMaximum(numblocks);

			for(int block = 0; enabled && block < numblocks; )
			{
				try
				{
					index = block * blocksize;
					cw = 0;

					if(block == lastblock && source.length % blocksize != 0)
					{
						blocksize = source.length % blocksize;
					}

					for(y = 0; y < ROWS_IN_BLOCK; y++)
					{
						for(x = 0; x < COLS_IN_BLOCK; x++)
						{
							chars[y][x] = source[index++];
							cws[y][x] = metrics.charWidth(chars[y][x]);

							if(cws[y][x] > cw)
							{
								cw = cws[y][x];
							}

							if(index >= source.length)
							{
								break;
							}
						}

						if(index >= source.length)
						{
							break;
						}
					}

					width = cw * COLS_IN_BLOCK;

					image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
					g = image.createGraphics();

					if(clearType != null)
					{
						g.setRenderingHints(clearType);
					}

					g.setColor(Color.BLACK);
					g.fillRect(0, 0, width, height);

					g.setColor(Color.WHITE);
					g.setFont(font);

					if(charset < 0)
					{
						imgname = Integer.toHexString(block) + ".png";
					}
					else
					{
						imgname = Encoder.cpnames[charset] + ".png";
					}

					ps.println("[" + imgname + "]");
					ps.println();
					
					binout.writeUTF(imgname);
					binout.writeShort(blocksize);

					index = 0;

					for(y = 0; y < ROWS_IN_BLOCK; y++)
					{
						for(x = 0; x < COLS_IN_BLOCK; x++)
						{
							cx = x * cw;
							cy = y * fh;

							c[0] = chars[y][x];

							g.setClip(cx, cy, cw, fh);
							g.drawChars(c, 0, 1, cx, cy + offset);

							buf.setLength(0);

							buf.append(Integer.toHexString(c[0]).toUpperCase());
							buf.append("\t=\t");

							//buf.append(imgname);
							//buf.append("\t@\t");

							buf.append(cx);
							buf.append(",\t");
							buf.append(cy);
							buf.append(",\t");
							buf.append(cws[y][x]);

							//buf.append(",\t");
							//buf.append(fh);

							//buf.append("\t;\t");
							//buf.append((char)chars[y][x]);

							ps.println(buf.toString());
							
							binout.writeShort(c[0]);
							
							hi = c[0] >>> 8;
							lo = c[0] & 0xFF;
							
							srcindex[hi][lo] = (short)block;
							srcx[hi][lo] = (short)cx;
							srcy[hi][lo] = (short)cy;
							widths[hi][lo] = (short)cws[y][x];

							if(++index >= blocksize)
							{
								break;
							}
						}

						if(index >= blocksize)
						{
							break;
						}
					}

					ps.println();

					if(charset == CHARSET_UNICODE)
					{
						lPreview.setIcon(new ImageIcon(image));
					}

					png.reset();
					PNGFile.writePNG(image, pngout, PNGFile.GREYSCALE, PNGFile.FILTER_NONE, this);
					pngout.flush();

					zip.putNextEntry(new ZipEntry(imgname));
					zip.write(png.toByteArray());
				}
				catch(Throwable t)
				{
					System.out.println("Error while processing block " + block);
					t.printStackTrace();
				}

				block++;

				pbTotal.setValue(block);
			}

			ps.flush();

			zip.putNextEntry(new ZipEntry("font.ini"));
			zip.write(ini.toByteArray());

			ps.close();
			
			for(hi = 0; hi < 256; hi++)
			{
				for(lo = 0; lo < 256; lo++)
				{
					binout.writeShort(srcindex[hi][lo]);

					if(srcindex[hi][lo] >= 0)
					{
						binout.writeShort(srcx[hi][lo]);
						binout.writeShort(srcy[hi][lo]);
						binout.writeShort(widths[hi][lo]);
					}
				}
			}

			binout.flush();

			zip.putNextEntry(new ZipEntry("font.bin"));
			zip.write(bin.toByteArray());

			binout.close();

			zip.close();
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}

		thread = null;

		btnStart.setText("Start");
		
		pbTotal.setValue(0);
		pbEncode.setValue(0);

		updatePreview();
	}

	public void setMax(int max)
	{
		pbEncode.setMaximum(max);
	}

	public void setProgress(int progress)
	{
		pbEncode.setValue(progress);
	}

	public void rise(int plus)
	{
		pbEncode.setMaximum(pbEncode.getMaximum() + plus);
	}

	public void progress(int plus)
	{
		pbEncode.setValue(pbEncode.getValue() + plus);
	}

	public void setText(String text)
	{
	}

	public int getMax()
	{
		return pbEncode.getMaximum();
	}

	public int getProgress()
	{
		return pbEncode.getValue();
	}

	public String getText()
	{
		return "";
	}

	public void setPercentMode(boolean mode)
	{
	}

	public void setCancelFlag(boolean flag)
	{
	}

	public boolean getCancelFlag()
	{
		return false;
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        cbFamily = new javax.swing.JComboBox();
        spSize = new javax.swing.JSpinner();
        chBold = new javax.swing.JCheckBox();
        chItalic = new javax.swing.JCheckBox();
        chClearType = new javax.swing.JCheckBox();
        jPanel2 = new javax.swing.JPanel();
        tfSample = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        lPreview = new javax.swing.JLabel();
        btnBackColor = new javax.swing.JButton();
        btnForeColor = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        tfPath = new javax.swing.JTextField();
        btnStart = new javax.swing.JButton();
        cbCharset = new javax.swing.JComboBox();
        jPanel4 = new javax.swing.JPanel();
        pbTotal = new javax.swing.JProgressBar();
        pbEncode = new javax.swing.JProgressBar();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("FontCreator");

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Font"));

        cbFamily.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbFamilyItemStateChanged(evt);
            }
        });

        spSize.setModel(new javax.swing.SpinnerNumberModel(16, 1, 72, 1));
        spSize.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spSizeStateChanged(evt);
            }
        });

        chBold.setText("Bold");
        chBold.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                chBoldItemStateChanged(evt);
            }
        });

        chItalic.setText("Italic");
        chItalic.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                chItalicItemStateChanged(evt);
            }
        });

        chClearType.setText("ClearType");
        chClearType.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        chClearType.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
        chClearType.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                chClearTypeItemStateChanged(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(chBold)
                        .addGap(18, 18, 18)
                        .addComponent(chItalic))
                    .addComponent(cbFamily, 0, 274, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(chClearType)
                    .addComponent(spSize, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cbFamily, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(spSize, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(9, 9, 9)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(chBold)
                    .addComponent(chItalic)
                    .addComponent(chClearType))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Preview"));

        tfSample.setText("Sample Text");
        tfSample.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                tfSampleKeyReleased(evt);
            }
        });

        lPreview.setBackground(new java.awt.Color(208, 208, 208));
        lPreview.setForeground(new java.awt.Color(0, 0, 80));
        lPreview.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        lPreview.setAutoscrolls(true);
        lPreview.setOpaque(true);
        jScrollPane1.setViewportView(lPreview);

        btnBackColor.setText("Bg");
        btnBackColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnBackColorActionPerformed(evt);
            }
        });

        btnForeColor.setText("Fg");
        btnForeColor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnForeColorActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addComponent(tfSample, javax.swing.GroupLayout.DEFAULT_SIZE, 274, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(btnForeColor)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnBackColor)))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(tfSample, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnBackColor)
                    .addComponent(btnForeColor))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 96, Short.MAX_VALUE)
                .addContainerGap())
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("File"));

        tfPath.setText(System.getProperty("user.home"));

        btnStart.setText("Start");
        btnStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnStartActionPerformed(evt);
            }
        });

        cbCharset.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Full Unicode", "All code pages" }));
        cbCharset.setSelectedIndex(1);
        cbCharset.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbCharsetItemStateChanged(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(tfPath, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel3Layout.createSequentialGroup()
                        .addComponent(cbCharset, 0, 274, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(btnStart, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(tfPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnStart)
                    .addComponent(cbCharset, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("Progress"));

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(pbTotal, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)
                    .addComponent(pbEncode, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addComponent(pbTotal, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pbEncode, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel4, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void cbFamilyItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_cbFamilyItemStateChanged
	{//GEN-HEADEREND:event_cbFamilyItemStateChanged
		updatePreview();
	}//GEN-LAST:event_cbFamilyItemStateChanged

	private void chBoldItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_chBoldItemStateChanged
	{//GEN-HEADEREND:event_chBoldItemStateChanged
		updatePreview();
	}//GEN-LAST:event_chBoldItemStateChanged

	private void chItalicItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_chItalicItemStateChanged
	{//GEN-HEADEREND:event_chItalicItemStateChanged
		updatePreview();
	}//GEN-LAST:event_chItalicItemStateChanged

	private void spSizeStateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_spSizeStateChanged
	{//GEN-HEADEREND:event_spSizeStateChanged
		updatePreview();
	}//GEN-LAST:event_spSizeStateChanged

	private void chClearTypeItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_chClearTypeItemStateChanged
	{//GEN-HEADEREND:event_chClearTypeItemStateChanged
		updatePreview();
	}//GEN-LAST:event_chClearTypeItemStateChanged

	private void tfSampleKeyReleased(java.awt.event.KeyEvent evt)//GEN-FIRST:event_tfSampleKeyReleased
	{//GEN-HEADEREND:event_tfSampleKeyReleased
		updatePreview();
	}//GEN-LAST:event_tfSampleKeyReleased

	private void btnStartActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnStartActionPerformed
	{//GEN-HEADEREND:event_btnStartActionPerformed
		if(thread == null)
		{
			enabled = true;

			thread = new Thread(this);
			thread.start();
		}
		else
		{
			enabled = false;

			try
			{
				if(thread != null && thread.isAlive())
				{
					thread.join();
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}//GEN-LAST:event_btnStartActionPerformed

	private void cbCharsetItemStateChanged(java.awt.event.ItemEvent evt)//GEN-FIRST:event_cbCharsetItemStateChanged
	{//GEN-HEADEREND:event_cbCharsetItemStateChanged
		updatePreview();
	}//GEN-LAST:event_cbCharsetItemStateChanged

	private void btnForeColorActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnForeColorActionPerformed
	{//GEN-HEADEREND:event_btnForeColorActionPerformed
		ColorListener listener = new ColorListener()
		{
			public void colorSelected(Color color)
			{
				lPreview.setForeground(color);
				updatePreview();
			}
		};
		
		(new ColorSelector(lPreview.getForeground(), listener)).setVisible(true);
	}//GEN-LAST:event_btnForeColorActionPerformed

	private void btnBackColorActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnBackColorActionPerformed
	{//GEN-HEADEREND:event_btnBackColorActionPerformed
		ColorListener listener = new ColorListener()
		{
			public void colorSelected(Color color)
			{
				lPreview.setBackground(color);
				updatePreview();
			}
		};

		(new ColorSelector(lPreview.getBackground(), listener)).setVisible(true);
	}//GEN-LAST:event_btnBackColorActionPerformed

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[])
	{
		java.awt.EventQueue.invokeLater(new Runnable()
		{
			public void run()
			{
				new MainFrame().setVisible(true);
			}
		});
	}
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnBackColor;
    private javax.swing.JButton btnForeColor;
    private javax.swing.JButton btnStart;
    private javax.swing.JComboBox cbCharset;
    private javax.swing.JComboBox cbFamily;
    private javax.swing.JCheckBox chBold;
    private javax.swing.JCheckBox chClearType;
    private javax.swing.JCheckBox chItalic;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel lPreview;
    private javax.swing.JProgressBar pbEncode;
    private javax.swing.JProgressBar pbTotal;
    private javax.swing.JSpinner spSize;
    private javax.swing.JTextField tfPath;
    private javax.swing.JTextField tfSample;
    // End of variables declaration//GEN-END:variables
}
